%{
  /* Counter, in which line a word was found. */
  unsigned int lineCounter = 1;

  /* Forward declaration of helper function. */
  void incrementLineCounter();
%}

digit [0-9]
letter [a-zA-Z]
identifier {letter}({letter}|{digit})*


%%


 /* Matches integer numbers.
    yytext is a pointer to the matched string. Rules start in column 0. Any
    indented parts are directly copied to the output file. */
{digit}+	printf( "%d: number %d\n", lineCounter, atoi( yytext ) );

 /* Matches operators.
    Longer C code snippets for actions can be enclosed in { ... } */
"+"|"++"|"-"|"--"|"*"|"/"|"="|"==" {
  printf( "%d: operator %s\n", lineCounter, yytext );
}

 /* Matches keywords of some fictive programming language. */
if|then|else|for|while|do|to {
  printf( "%d: keyword %s\n", lineCounter, yytext );
}

 /* Matches identifiers. Used names of definitions in a rule's pattern must be
    enclosed in { ... } */
{identifier}	printf( "%d: identifier %s\n", lineCounter, yytext );

[ \t]		/* Consumes white spaces and tabs. */

 /* Consumes new-line characters. Increments line counter. */
\n		incrementLineCounter();


%%


/* Increment global variable lineCounter by 1. */
void incrementLineCounter()
{
  lineCounter++;
}


/* A complete C program needs a main function. */
int main()
{
  yylex();			/* Invoke scanner. */
  printf( "Read lines: %d\n", lineCounter );
  return 0;
}
